# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django import template
from django.utils.translation import gettext_lazy as _

from mailmanclient import MailingList

from postorius.auth.utils import user_is_in_list_roster
from postorius.forms.fields import DELIVERY_MODE_CHOICES
from postorius.forms.list_forms import ACTION_CHOICES
from postorius.models import List


register = template.Library()

__all__ = [
    'get_list',
    'user_is_list_moderator',
    'user_is_list_owner',
]


def get_list(mlist):
    """
    Given either a mailing list identifier or MailingList object itself, return
    the MailingList object. Identifiers could be one of the following:
    - List's posting address: test_list@example.com
    - List's fqdn: test_list.example.com
    """
    return mlist if isinstance(mlist, MailingList) else List.objects.get(mlist)


@register.simple_tag
def user_is_list_owner(user, mlist):
    """
    Given a User object and a MailingList object/identifier, returns True if
    the user is an owner of the given MailingList False otherwise.
    """
    return user_is_in_list_roster(user, get_list(mlist), 'owner')


@register.simple_tag
def user_is_list_moderator(user, mlist):
    """
    Given a User object and a MailingList object/identifier, return True if
    the user is one of the list moderators, False otherwise.
    """
    return user_is_in_list_roster(user, get_list(mlist), 'moderator')


@register.filter
def owner_repr(owner):
    name = owner.display_name or ''
    address = owner.addresses[0].original_email
    return '{} {}'.format(name, address)


@register.filter
def delivery_mode(value):
    """Convert the delivery_mode value into translated value."""
    return dict(DELIVERY_MODE_CHOICES).get(value, _('None'))


@register.filter
def moderation_action(value):
    """Convert the moderation_action value into translated text."""
    return dict(ACTION_CHOICES).get(value, _('List default'))



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


import logging

from django.shortcuts import render
from django.utils.translation import gettext_lazy as _

from allauth.account.models import EmailAddress
from django_mailman3.lib import mailman


logger = logging.getLogger(__name__)


def render_api_error(request):
    """Renders an error template.
    Use if MailmanApiError is catched.
    """
    return render(
        request,
        'postorius/errors/generic.html',
        {
            'error': _(
                'Mailman REST API not available. Please start Mailman core.'
            )
        },  # noqa: E501
        status=503,
    )


def render_client_error(request, error):
    return render(
        request,
        'postorius/errors/generic.html',
        {'error': str(error)},
        status=error.code,
    )


def get_mailman_client():
    """A proxy for django_mailman3.lib.mailman.get_mailman_client."""
    return mailman.get_mailman_client()


def with_empty_choice(choices):
    """Add an empty Choice for unset values in dropdown."""
    return [(None, '-----')] + list(choices)


def set_preferred(user, mm_user):
    """Set preferred address in Mailman Core.

    :param user: The Django user mode to set preferred address.
    :param mm_user: The Mailman User object to set preferred address for.
    """
    client = get_mailman_client()
    primary_email = EmailAddress.objects.get_primary(user)
    if primary_email is not None and primary_email.verified:
        # First, make sure that the email address is verified in Core,
        # otherwise, we can't set it as a primary address.
        addr = client.get_address(primary_email.email)
        if not addr.verified_on:
            addr.verify()
        mm_user.preferred_address = primary_email.email
        return primary_email.email
    return None


def get_member_or_nonmember(mlist, email):
    """Return either a Member or a Non-member with `email` in mlist.

    :param mlist: MailingList object to get membership for.
    :param email: Email address of the member or nonmember.
    :returns: Member if found otherwise None.
    """
    try:
        member = mlist.get_member(email)
    except ValueError:
        # Not a Member, try getting non-member.
        try:
            member = mlist.get_nonmember(email)
        except ValueError:
            member = None
    return member


def get_django_user(mm_user, addresses=None):
    """Given a Mailman user, return a Django User if One exists.

    There is no direct way to fetch that, but we can iterate on Users's Email
    Addresses and see if Django knows any one of them. If we find one, just
    return the user associated with that Address.

    Ideally, the email addresses should be *all* synchronized with Core, but
    just in case they aren't, we iterate over all the addresses.

    :param mm_user: Mailman User object.
    :param addresses: user.addresses for the above mailman objects. It is
        passed in just so that we can avoid an API call to Core to get them. It
        is optional since we can just fetch them here if need be.
    :returns: Django user if found, None otherwise.
    """
    if addresses is None:
        addresses = mm_user.addresses or []
    django_email = None
    for addr in addresses:
        try:
            django_email = EmailAddress.objects.get(email=addr.email)
        except EmailAddress.DoesNotExist:
            continue
        else:
            break

    if django_email is None:
        # If none of the user's emails are registered in Django, that means
        # they don't have a user account.
        return None
    return django_email.user


def filter_memberships_by_roles(memberships, roles):
    """Given a list of roles, filter the memberships with those roles.

    :param memberships: A list of Member objects.
    :type memberships: List[mailmanclient.restobjects.Member]
    :param roles: A list of roles.
    :type roles: List[str]
    :returns: A list of memberships filtered by roles.
    :rtype: List[mailmanclient.restobjects.Member]
    """
    return [member for member in memberships if member.role in roles]


LANGUAGES = [
    ('ar', 'Arabic'),
    ('ast', 'Asturian'),
    ('bg', 'Bulgarian'),
    ('ca', 'Catalan'),
    ('zh_CN', 'Chinese'),
    ('zh_TW', 'Chinese (Taiwan)'),
    ('hr', 'Croatian'),
    ('cs', 'Czech'),
    ('da', 'Danish'),
    ('nl', 'Dutch'),
    ('en', 'English (USA)'),
    ('et', 'Estonian'),
    ('eu', 'Euskara'),
    ('fi', 'Finnish'),
    ('fr', 'French'),
    ('gl', 'Galician'),
    ('de', 'German'),
    ('el', 'Greek'),
    ('he', 'Hebrew'),
    ('hu', 'Hungarian'),
    ('ia', 'Interlingua'),
    ('it', 'Italian'),
    ('ja', 'Japanese'),
    ('ko', 'Korean'),
    ('lt', 'Lithuanian'),
    ('no', 'Norwegian'),
    ('pl', 'Polish'),
    ('pt', 'Portuguese'),
    ('pt_BR', 'Portuguese (Brazil)'),
    ('ro', 'Romanian'),
    ('ru', 'Russian'),
    ('sr', 'Serbian'),
    ('sk', 'Slovak'),
    ('sl', 'Slovenian'),
    ('es', 'Spanish'),
    ('sv', 'Swedish'),
    ('tr', 'Turkish'),
    ('uk', 'Ukrainian'),
    ('vi', 'Vietnamese'),
]



# -*- coding: utf-8 -*-
# Copyright (C) 2019-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
from django import template
from django.utils.dateparse import parse_datetime


register = template.Library()


@register.filter
def datetime_parse(value):
    """Parse string value into datetime object."""
    try:
        return parse_datetime(value)
    except ValueError:
        # If they are invalid dates, just return them back.
        return value



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

import logging
from urllib.error import HTTPError

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import Site
from django.http import Http404
from django.shortcuts import redirect, render
from django.utils.translation import gettext_lazy as _
from django.views.decorators.http import require_POST

from django_mailman3.lib.mailman import get_mailman_client
from django_mailman3.models import MailDomain
from django_mailman3.signals import domain_created, domain_deleted

from postorius.auth.decorators import superuser_required
from postorius.forms.domain_forms import (
    DomainEditForm,
    DomainForm,
    DomainOwnerForm,
)
from postorius.models import Domain, Mailman404Error


log = logging.getLogger(__name__)


@login_required
@superuser_required
def domain_index(request):
    existing_domains = Domain.objects.all()
    for domain in existing_domains:
        try:
            web_host = MailDomain.objects.get(mail_domain=domain.mail_host)
        except MailDomain.DoesNotExist:
            site = Site.objects.get_current(request)
            web_host = MailDomain.objects.create(
                site=site, mail_domain=domain.mail_host
            )
        domain.site = web_host.site
    return render(
        request,
        'postorius/domain/index.html',
        {
            'domains': existing_domains,
        },
    )


@login_required
@superuser_required
def domain_new(request):
    form_initial = {'site': Site.objects.get_current(request)}
    if request.method == 'POST':
        form = DomainForm(request.POST, initial=form_initial)
        if form.is_valid():
            domain = Domain(
                mail_host=form.cleaned_data['mail_host'],
                description=form.cleaned_data['description'],
                alias_domain=form.cleaned_data['alias_domain'],
                owner=request.user.email,
            )
            try:
                domain.save()
            except HTTPError as e:
                form.add_error('mail_host', e.reason)
            else:
                messages.success(request, _('New Domain registered'))
                MailDomain.objects.get_or_create(
                    site=form.cleaned_data['site'],
                    mail_domain=form.cleaned_data['mail_host'],
                )
                domain_created.send(
                    sender=Domain, mail_host=form.cleaned_data['mail_host']
                )
                return redirect('domain_index')
    else:
        form = DomainForm(initial=form_initial)
    return render(request, 'postorius/domain/new.html', {'form': form})


@login_required
@superuser_required
def domain_edit(request, domain):
    try:
        domain_obj = Domain.objects.get(mail_host=domain)
    except Mailman404Error:
        raise Http404('Domain does not exist')
    if request.method == 'POST':
        form = DomainEditForm(request.POST)
        if form.is_valid():
            domain_obj.description = form.cleaned_data['description']
            domain_obj.alias_domain = form.cleaned_data['alias_domain']
            try:
                web_host = MailDomain.objects.get(mail_domain=domain)
            except MailDomain.DoesNotExist:
                web_host = MailDomain.objects.create(
                    site=form.cleaned_data['site'], mail_domain=domain
                )
            else:
                web_host.site = form.cleaned_data['site']
                web_host.save()
            try:
                domain_obj.save()
            except HTTPError as e:
                messages.error(request, e)
                return redirect('domain_edit', domain=domain)
            else:
                messages.success(request, _('Domain %s updated') % domain)
                return redirect('domain_index')
        else:
            messages.error(request, _('Please check the errors below'))
    else:
        form_initial = {
            'description': domain_obj.description,
            'alias_domain': domain_obj.alias_domain,
            'site': MailDomain.objects.get(mail_domain=domain).site,
        }
        form = DomainEditForm(initial=form_initial)

    return render(
        request, 'postorius/domain/edit.html', {'domain': domain, 'form': form}
    )


@login_required
@superuser_required
def domain_delete(request, domain):
    """Deletes a domain but asks for confirmation first."""
    domain_obj = Domain.objects.get(mail_host=domain)
    if request.method == 'POST':
        try:
            domain_obj.delete()
            MailDomain.objects.filter(mail_domain=domain).delete()
            messages.success(
                request, _('The domain %s has been deleted.' % domain)
            )
            domain_deleted.send(sender=Domain, mail_host=domain)
            return redirect('domain_index')
        except HTTPError as e:
            messages.error(
                request, _('The domain could not be deleted: %s' % e.msg)
            )
            return redirect('domain_index')
    domain_lists_page = domain_obj.get_list_page(count=10)
    return render(
        request,
        'postorius/domain/confirm_delete.html',
        {'domain': domain_obj, 'lists': domain_lists_page},
    )


@login_required
@superuser_required
def domain_owners(request, domain):
    domain_obj = Domain.objects.get(mail_host=domain)
    if request.method == 'POST':
        form = DomainOwnerForm(request.POST)
        if form.is_valid():
            email = form.cleaned_data['email']
            domain_obj.add_owner(email)
            messages.success(
                request,
                _('Added {} as an owner for {}').format(
                    email, domain_obj.mail_host
                ),
            )
            return redirect('domain_index')
    else:
        form = DomainOwnerForm()
    return render(
        request,
        'postorius/domain/owners.html',
        {'domain': domain_obj, 'form': form},
    )


@require_POST
@login_required
@superuser_required
def remove_owners(request, domain, user_id):
    domain_obj = Domain.objects.get(mail_host=domain)
    # Since there is no way to remove one single owner, we do the only possible
    # thing, remove all owners and add the rest back.
    client = get_mailman_client()
    try:
        remove_email = client.get_user(user_id).addresses[0].email
        all_owners_emails = [
            owner.addresses[0].email for owner in domain_obj.owners
        ]
    except (KeyError, ValueError) as e:
        # We get KeyError if the user has no address due to [0].
        log.error('Unable to delete owner: %s', str(e))
        raise Http404(str(e))
    if remove_email in all_owners_emails:
        all_owners_emails.remove(remove_email)
    else:
        messages.error(
            _('{} is not an owner for {}').format(
                remove_email, domain_obj.mail_host
            )
        )
        return redirect('domain_index')
    domain_obj.remove_all_owners()
    for owner in all_owners_emails:
        domain_obj.add_owner(owner)
    messages.success(
        request,
        _('Removed {} as an owner for {}').format(
            remove_email, domain_obj.mail_host
        ),
    )
    return redirect('domain_index')



# -*- coding: utf-8 -*-
# Copyright (C) 2015-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.

import logging
from urllib.error import HTTPError

from django_mailman3.lib import mailman
from mailmanclient import MailmanConnectionError

from postorius import utils
from postorius.models import MailmanApiError


logger = logging.getLogger('postorius')


__all__ = [
    'PostoriusMiddleware',
]


class PostoriusMiddleware(object):
    def __init__(self, get_response=None):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_exception(self, request, exception):
        if isinstance(exception, (MailmanApiError, MailmanConnectionError)):
            logger.exception('Mailman REST API not available')
            return utils.render_api_error(request)
        elif isinstance(exception, HTTPError):
            logger.exception('Un-handled exception: %s', str(exception))
            return utils.render_client_error(request, exception)


class APICountingMiddleware:
    """Counts the total number of API calls made to Core and prints summary.

    It hooks into mailmanclient's hooking mechanism, facilitated by
    ``django_mailman3.lib.mailman.mailmanclient_request_hook``. It stores the
    parameter of each API call made in `self.api_call_counter()` and prints it
    *before* returning the response in `__call__`.

    It hooks up ``_trace_api_calls()`` method into the Hooks API. Even if the
    hooking happens multiple times, it is only added once since it
    deduplicates the hooks.

    Output looks something like:

    DEBUG: =======================
    DEBUG: Handle reqsponse for /postorius/lists/mylist.lists.araj.me/settings/
    DEBUG: View function was postorius.views.list.list_settings
    DEBUG: 4 calls to API
    DEBUG: [GET] http://localhost:8001/3.1/lists/mylist.lists.araj.me with None
    DEBUG: [snip]
    DEBUG: ======================

    Note: If you don't see the output on console, check the 'postorius' logger
    settings and make sure that it has `console` handler and level is set to
    'DEBUG'.

    """

    def __init__(self, get_response=None):
        self.get_response = get_response
        mailman.mailmanclient_request_hook(self._trace_api_calls)
        self.api_call_counter = []
        self.view_func = ''

    def __call__(self, request):
        self.request = request
        response = self.get_response(request)
        logger.debug('=======================')
        logger.debug('Handle reqsponse for %s', request.path)
        logger.debug('View function was %s', self.view_func)
        logger.debug('%s calls to API', len(self.api_call_counter))
        for each in self.api_call_counter:
            logger.debug(
                '[%s] %s with %s',
                each.get('method'),
                each.get('url'),
                each.get('data'),
            )
        logger.debug('=======================')
        self.api_call_counter = []
        return response

    def _trace_api_calls(self, params):
        """Hook that adds all the call parameters to self.api_call_counter.

        :param params: List of request params from mailmanclient.
        """
        self.api_call_counter.append(params)
        return params

    def process_view(self, request, view_func, view_args, view_kwars):
        """Get a pointer to view function object."""
        self.view_func = '{}.{}'.format(
            view_func.__module__, view_func.__name__
        )



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.contrib.auth.decorators import login_required
from django.shortcuts import render
from django.utils.translation import gettext_lazy

from django_mailman3.lib.mailman import get_mailman_client

from postorius.auth.decorators import superuser_required
from postorius.views.generic import bans_view


SYSTEM_INFO_KEYS = (
    ('mailman_version', gettext_lazy('Mailman Core Version')),
    ('api_version', gettext_lazy('Mailman Core API Version')),
    ('python_version', gettext_lazy('Mailman Core Python Version')),
)


@login_required
@superuser_required
def system_information(request):
    client = get_mailman_client()
    all_configs = client.system

    configs = []
    for key, name in SYSTEM_INFO_KEYS:
        configs.append((name, all_configs.get(key)))

    queues = {
        queue_name: len(queue.files)
        for queue_name, queue in client.queues.items()
    }

    return render(
        request,
        'postorius/system_information.html',
        {'configs': configs, 'queues': queues},
    )


@login_required
@superuser_required
def bans(request):
    return bans_view(request, template='postorius/bans.html')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-07-12 05:36
from __future__ import unicode_literals

from django.db import migrations, models


# flake8: noqa


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0006_auto_20180711_1359'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='language',
            field=models.CharField(blank=True, choices=[('ar', 'Arabic'), ('ast', 'Asturian'), ('ca', 'Catalan'), ('cs', 'Czech'), ('da', 'Danish'), ('de', 'German'), ('el', 'Greek'), ('es', 'Spanish'), ('et', 'Estonian'), ('eu', 'Euskara'), ('fi', 'Finnish'), ('fr', 'French'), ('gl', 'Galician'), ('he', 'Hebrew'), ('hr', 'Croatian'), ('hu', 'Hungarian'), ('ia', 'Interlingua'), ('it', 'Italian'), ('ja', 'Japanese'), ('ko', 'Korean'), ('lt', 'Lithuanian'), ('nl', 'Dutch'), ('no', 'Norwegian'), ('pl', 'Polish'), ('pt', 'Protuguese'), ('pt_BR', 'Protuguese (Brazil)'), ('ro', 'Romanian'), ('ru', 'Russian'), ('sk', 'Slovak'), ('sl', 'Slovenian'), ('sr', 'Serbian'), ('sv', 'Swedish'), ('tr', 'Turkish'), ('uk', 'Ukrainian'), ('vi', 'Vietnamese'), ('zh_CN', 'Chinese'), ('zh_TW', 'Chinese (Taiwan)'), ('en', 'English (USA)')], help_text="Language for the template, this should be the list's preferred language.", max_length=5),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2018-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
# flake8: noqa

from django.utils.translation import gettext_lazy as _


TEMPLATES_LIST = (
    (
        'list:admin:action:post',
        _(
            'Sent to the list administrators when moderator approval for a posting is required.'
        ),
    ),
    (
        'list:admin:action:subscribe',
        _(
            'Sent to the list administrators when moderator approval for a subscription request is required.'
        ),
    ),
    (
        'list:admin:action:unsubscribe',
        _(
            'Sent to the list administrators when moderator approval for an unsubscription request is required.'
        ),
    ),
    (
        'list:admin:notice:subscribe',
        _(
            'Sent to the list administrators to notify them when a new member has been subscribed.'
        ),
    ),
    (
        'list:admin:notice:unrecognized',
        _(
            'Sent to the list administrators when a bounce message in an unrecognized format has been received.'
        ),
    ),
    (
        'list:admin:notice:pending',
        _(
            "Sent to the list administrators from the 'mailman notify' command."
        ),
    ),
    (
        'list:admin:notice:increment',
        _(
            "Sent to the list administrators to notify them when a member's bounce score is incremented."
        ),
    ),
    (
        'list:admin:notice:disable',
        _(
            "Sent to the list administrators to notify them when a member's delivery is disabled due to excessive bounces."
        ),
    ),
    (
        'list:admin:notice:unsubscribe',
        _(
            'Sent to the list administrators to notify them when a member has been unsubscribed.'
        ),
    ),
    ('list:member:digest:footer', _('The footer for a digest message.')),
    ('list:member:digest:header', _('The header for a digest message.')),
    (
        'list:member:digest:masthead',
        _(
            'The digest “masthead”; i.e. a common introduction for all digest messages.'
        ),
    ),
    (
        'list:member:regular:footer',
        _('The footer for a regular (non-digest) message.'),
    ),
    (
        'list:member:regular:header',
        _('The header for a regular (non-digest) message.'),
    ),
    (
        'list:user:action:subscribe',
        _(
            'The message sent to subscribers when a subscription confirmation is required.'
        ),
    ),
    (
        'list:user:action:unsubscribe',
        _(
            'The message sent to subscribers when an unsubscription confirmation is required.'
        ),
    ),
    (
        'list:user:action:invite',
        _(
            'The message sent to subscribers when they are invited to join a list.'
        ),
    ),
    (
        'list:user:notice:goodbye',
        _(
            'The notice sent to a member when they unsubscribe from a mailing list.'
        ),
    ),
    (
        'list:user:notice:hold',
        _(
            'The notice sent to a poster when their message is being held for moderator approval.'
        ),
    ),
    (
        'list:user:notice:no-more-today',
        _(
            'Sent to a user when the maximum number of autoresponses has been reached for that day.'
        ),
    ),
    (
        'list:user:notice:post',
        _(
            'Notice sent to a poster when their message has been received by the mailing list.'
        ),
    ),
    (
        'list:user:notice:probe',
        _(
            'A bounce probe sent to a member when their subscription has been disabled due to bounces.'
        ),
    ),
    (
        'list:user:notice:refuse',
        _(
            'Notice sent to a poster when their message has been rejected by the list’s moderator.'
        ),
    ),
    (
        'list:user:notice:rejected',
        _(
            'Notice sent to a poster when their message has been rejected automatically.'
        ),
    ),
    (
        'list:user:notice:welcome',
        _(
            'The notice sent to a member when they are subscribed to the mailing list.'
        ),
    ),
    (
        'list:user:notice:warning',
        _(
            'The notice sent to a member when their membership has been disabled due to excessive bounces.'
        ),
    ),
)



# -*- coding: utf-8 -*-
# Copyright (C) 2017-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#


from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from allauth.account.models import EmailAddress

from postorius.forms.fields import (
    NullBooleanRadioSelect,
    delivery_mode_field,
    delivery_status_field,
    moderation_action_field,
)
from postorius.utils import LANGUAGES, with_empty_choice


class UserPreferences(forms.Form):

    """
    Form handling the user's global, address and subscription based preferences
    """

    def __init__(self, *args, **kwargs):
        self._preferences = kwargs.pop('preferences', None)
        self.disabled_delivery_choices = kwargs.pop(
            'disabled_delivery_choices', []
        )
        super().__init__(*args, **kwargs)
        # Disable some options to be set.
        self.fields[
            'delivery_status'
        ].widget.disabled_choices = self.disabled_delivery_choices

    @property
    def initial(self):
        # Redirect to the preferences, this allows setting the preferences
        # after instanciation and it will also set the initial data.
        return self._preferences or {}

    @initial.setter
    def initial(self, value):
        pass

    choices = ((True, _('Yes')), (False, _('No')))

    delivery_status = delivery_status_field()
    delivery_mode = delivery_mode_field()
    receive_own_postings = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Receive own postings'),
        help_text=_(
            'Ordinarily, you will get a copy of every message you post to the '
            "list. If you don't want to receive this copy, set this option "
            'to No.'
        ),
    )
    acknowledge_posts = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Acknowledge posts'),
        help_text=_(
            'Receive acknowledgement mail when you send mail to the list?'
        ),
    )
    hide_address = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Hide address'),
        help_text=_(
            'When other members allowed to view the list membership, '
            'your email address is normally shown. '
            'If you do not want your email address to show up on this '
            'membership roster, select Yes for this option.'
        ),
    )
    receive_list_copy = forms.NullBooleanField(
        widget=NullBooleanRadioSelect(choices=choices),
        required=False,
        label=_('Receive list copies (possible duplicates)'),
        help_text=_(
            'When you are listed explicitly in the To: or Cc: headers of a '
            'list message, you can opt to not receive another copy from the '
            'mailing list. Select Yes to receive copies. '
            'Select No to avoid receiving copies from the mailing list'
        ),
    )

    preferred_language = forms.ChoiceField(
        widget=forms.Select(),
        choices=with_empty_choice(LANGUAGES),
        required=False,
        label=_('Preferred language'),
        help_text=_(
            'Preferred language for your interactions with Mailman. When '
            "this is set, it will override the MailingList's preferred "
            'language. This affects which language is used for your '
            'email notifications and such.'
        ),
    )

    class Meta:

        """
        Class to define the name of the fieldsets and what should be
        included in each.
        """

        layout = [
            [
                'User Preferences',
                'acknowledge_posts',
                'hide_address',
                'receive_list_copy',
                'receive_own_postings',
                'delivery_mode',
                'delivery_status',
                'preferred_language',
            ]
        ]

    def save(self):
        # Note (maxking): It is possible that delivery_status field will always
        # be a part of changed_data because of how the SelectWidget() works.
        if not self.changed_data:
            return
        for key in self.changed_data:
            if self.cleaned_data[key] not in (None, ''):
                # None: nothing set yet. Remember to remove this test
                # when Mailman accepts None as a "reset to default"
                # value.
                self._preferences[key] = self.cleaned_data[key]
        self._preferences.save()

    def clean_delivery_status(self):
        """Check that someone didn't pass the disabled value.

        This is meant to enforce that certain values are RO and can be seen but
        not set.
        """
        val = self.cleaned_data.get('delivery_status')
        # When the options are disabled in the widget, the values returned are
        # empty. Consider that as unchanged values and just return the initial
        # value of the field.
        if not val:
            return self.initial.get('delivery_status')
        # This means the value was changed, check if the change was allowed. If
        # not, just raise a ValidationError.
        if val in self.disabled_delivery_choices:
            raise ValidationError(
                _('Cannot set delivery_status to {}').format(val)
            )
        # The change seems correct, just return the value.
        return val


class UserPreferencesFormset(forms.BaseFormSet):
    def __init__(self, *args, **kwargs):
        self._preferences = kwargs.pop('preferences')
        self._disabled_delivery_choices = kwargs.pop(
            'disabled_delivery_choices', []
        )
        kwargs['initial'] = self._preferences
        super(UserPreferencesFormset, self).__init__(*args, **kwargs)

    def _construct_form(self, i, **kwargs):
        form = super(UserPreferencesFormset, self)._construct_form(i, **kwargs)
        form._preferences = self._preferences[i]
        form.fields[
            'delivery_status'
        ].widget.disabled_choices = self._disabled_delivery_choices
        return form

    def save(self):
        for form in self.forms:
            form.save()


class ManageMemberForm(forms.Form):

    # RW fields.
    moderation_action = moderation_action_field()
    delivery_mode = delivery_mode_field()

    # TODO: Maybe add Member's preferences here to set other things like
    # delivery_mode and such?

    def __init__(self, *args, **kw):
        self.member = kw.pop('member')
        super().__init__(*args, **kw)

    @property
    def initial(self):
        return {
            'moderation_action': self.member.moderation_action,
            'delivery_mode': self.member.delivery_mode,
        }

    @initial.setter
    def initial(self, value):
        pass

    def save(self):
        """Save the data to the Member object by calling into REST API.

        Also, return True/False to determine if anything was updated or not.
        """
        if not self.changed_data:
            return False
        for each in self.changed_data:
            updated = self.cleaned_data.get(each)
            if updated not in (None, ''):
                setattr(self.member, each, updated)
        self.member.save()
        return True


class ManageMemberFormSet(forms.BaseFormSet):
    def __init__(self, *args, **kw):
        self._members = kw.pop('members')
        kw['initial'] = self._members
        super().__init__(*args, **kw)

    def get_form_kwargs(self, index):
        kwargs = super().get_form_kwargs(index)
        kwargs['member'] = self._members[index]
        return kwargs

    def _construct_form(self, i, **kwargs):
        form = super()._construct_form(i, **kwargs)
        form.member = self._members[i]
        return form

    def save(self):
        """Save and return the lists for which subs were updated"""
        updated = []
        for form in self.forms:
            was_updated = form.save()
            if was_updated:
                updated.append(form.member.list_id)
        return updated


class ManageAddressForm(forms.Form):
    # RW fields.
    verified = forms.BooleanField(
        widget=forms.CheckboxInput(),
        required=False,
        label=_('Verified'),
        help_text=_('Specifies whether or not this email address is verified'),
    )

    # TODO: Primary/Preferred Address. Needs integration with EmailAddress
    # model from Django/Allauth.

    def __init__(self, *args, **kw):
        self.address = kw.pop('address')
        super().__init__(*args, **kw)

    @property
    def initial(self):
        return {'verified': self.address.verified}

    @initial.setter
    def initial(self, value):
        pass

    def save(self):
        """Save the data and return if there was anything changed."""
        if not self.changed_data:
            return False
        # Since there is a single field, the below shouldn't raise KeyError. In
        # future when more fields are added, it can raise KeyError so make sure
        # to use .get() and not do anything if value is None.
        verified = self.cleaned_data['verified']
        if verified:
            self.address.verify()
            self._update_django_address(True)
            return True
        self.address.unverify()
        self._update_django_address(False)
        return True

    def _update_django_address(self, value):
        """Verify/Unverify the EmailAddress model in Allauth."""
        try:
            email = EmailAddress.objects.get(email=self.address.email)
        except EmailAddress.DoesNotExist:
            return
        email.verified = value
        email.save()


class ManageAddressFormSet(forms.BaseFormSet):
    def __init__(self, *args, **kw):
        self._addresses = kw.pop('addresses')
        kw['initial'] = self._addresses
        super().__init__(*args, **kw)

    def get_form_kwargs(self, index):
        kwargs = super().get_form_kwargs(index)
        kwargs['address'] = self._addresses[index]
        return kwargs

    def _construct_form(self, i, **kwargs):
        form = super()._construct_form(i, **kwargs)
        form.address = self._addresses[i]
        return form

    def save(self):
        """Save and return which addresses were updated."""
        updated = []
        for form in self.forms:
            was_updated = form.save()
            if was_updated:
                updated.append(form.address.email)
        return updated


class ManageUserForm(forms.Form):

    display_name = forms.CharField(label=_('Display Name'), required=True)

    def __init__(self, *args, **kw):
        self.user = kw.pop('user')
        super().__init__(*args, **kw)

    @property
    def initial(self):
        return {'display_name': self.user.display_name}

    @initial.setter
    def initial(self, value):
        pass

    def save(self):
        if not self.changed_data:
            return
        new_name = self.cleaned_data.get('display_name')
        self.user.display_name = new_name
        self.user.save()
        for addr in self.user.addresses:
            addr.display_name = new_name
            addr.save()



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django.conf import settings
from django.contrib import admin
from django.http import Http404
from django.urls import include, path, re_path, reverse_lazy
from django.views.defaults import server_error
from django.views.generic import RedirectView


def not_found(request):
    """A test view to return 404 error to test 400.html"""
    raise Http404('Page not Found.')


urlpatterns = [
    re_path(
        r'^$',
        RedirectView.as_view(url=reverse_lazy('list_index'), permanent=True),
    ),
    re_path(r'^postorius/', include('postorius.urls')),
    re_path(r'', include('django_mailman3.urls')),
    re_path(r'^accounts/', include('allauth.urls')),
    # Add some testing routes to test 400/500 error pages without having to
    # introduce errors.
    re_path(r'500/$', server_error),
    re_path(r'400/$', not_found),
    # Django admin
    re_path(r'^admin/', admin.site.urls),
]


if settings.DEBUG:
    import debug_toolbar

    urlpatterns = [
        path('__debug__/', include(debug_toolbar.urls)),
    ] + urlpatterns



# -*- coding: utf-8 -*-

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AddressConfirmationProfile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False,
                                        auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=254)),
                ('activation_key', models.CharField(max_length=40)),
                ('created', models.DateTimeField()),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL,
                                           on_delete=models.CASCADE)),
            ],
        ),
    ]



# Generated by Django 3.2.11 on 2022-02-19 04:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0016_auto_20210810_2157'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='language',
            field=models.CharField(
                blank=True,
                choices=[
                    ('ar', 'Arabic'),
                    ('ast', 'Asturian'),
                    ('ca', 'Catalan'),
                    ('zh_CN', 'Chinese'),
                    ('zh_TW', 'Chinese (Taiwan)'),
                    ('hr', 'Croatian'),
                    ('cs', 'Czech'),
                    ('da', 'Danish'),
                    ('nl', 'Dutch'),
                    ('en', 'English (USA)'),
                    ('et', 'Estonian'),
                    ('eu', 'Euskara'),
                    ('fi', 'Finnish'),
                    ('fr', 'French'),
                    ('gl', 'Galician'),
                    ('de', 'German'),
                    ('el', 'Greek'),
                    ('he', 'Hebrew'),
                    ('hu', 'Hungarian'),
                    ('ia', 'Interlingua'),
                    ('it', 'Italian'),
                    ('ja', 'Japanese'),
                    ('ko', 'Korean'),
                    ('lt', 'Lithuanian'),
                    ('no', 'Norwegian'),
                    ('pl', 'Polish'),
                    ('pt', 'Portuguese'),
                    ('pt_BR', 'Portuguese (Brazil)'),
                    ('ro', 'Romanian'),
                    ('ru', 'Russian'),
                    ('sr', 'Serbian'),
                    ('sk', 'Slovak'),
                    ('sl', 'Slovenian'),
                    ('es', 'Spanish'),
                    ('sv', 'Swedish'),
                    ('tr', 'Turkish'),
                    ('uk', 'Ukrainian'),
                    ('vi', 'Vietnamese'),
                ],
                help_text="Language for the template, this should be the list's preferred language.",  # noqa: E501
                max_length=5,
            ),
        ),
    ]



# -*- coding: utf-8 -*-
# Copyright (C) 2021-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.
#
from django import template


register = template.Library()


@register.filter
def user_display(user):
    """Return a User's formatted display_name and email."""
    addresses = user.addresses
    if user.display_name is not None and user.display_name != '':
        if len(addresses) > 0:
            # We can't use email.utils.formataddr because it will return an
            # RFC2047 encoded display name part for non-ascii.  We don't need
            # to decide about quoting the display name as we aren't using this
            # as an RFC5322 address.
            return '{} <{}>'.format(user.display_name, addresses[0].email)
        else:
            return user.display_name
    if len(addresses) > 0:
        return addresses[0].email
    return 'None'



import uuid

from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.utils.translation import gettext_lazy as _


def validate_uuid_or_email(input_value):
    """Check if the value is a valid email or UUID.

    This is useful to determine if a given input is a user identifier in
    Mailman 3 API 3.1+, where, user identifiers are either UUID or email.

    :param input_value: The value to validate.
    :type input_value: str
    :returns: input_value
    :rtype: str
    :raises ValidationError: If the value is either a UUID, not an email.
    """

    try:
        validate_email(input_value)
    except ValidationError:
        is_email = False
    else:
        is_email = True

    is_uuid = False
    if not is_email:
        try:
            uuid.UUID(input_value)
        except ValueError:
            is_uuid = False
        else:
            is_uuid = True

    if not any([is_email, is_uuid]):
        raise ValidationError(
            _('Invalid: "{0}" should be either email or UUID').format(
                input_value
            ),
            params={'value': input_value},
            code='invalid',
        )
    return input_value



# -*- coding: utf-8 -*-
# Copyright (C) 1998-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


import logging
from urllib.error import HTTPError

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import AdminPasswordChangeForm
from django.forms import formset_factory
from django.http import Http404
from django.shortcuts import redirect, render
from django.urls import reverse, reverse_lazy
from django.utils.decorators import method_decorator
from django.utils.translation import gettext_lazy as _
from django.views.decorators.debug import sensitive_post_parameters
from django.views.decorators.http import require_GET
from django.views.generic import FormView

from allauth.account.models import EmailAddress
from django_mailman3.lib.mailman import get_mailman_client, get_mailman_user
from django_mailman3.lib.paginator import MailmanPaginator, paginate

from postorius.auth.decorators import superuser_required
from postorius.forms import (
    ChangeSubscriptionForm,
    ManageAddressForm,
    ManageAddressFormSet,
    ManageMemberForm,
    ManageMemberFormSet,
    ManageUserForm,
    UserPreferences,
    UserPreferencesFormset,
)
from postorius.models import List, SubscriptionMode
from postorius.utils import (
    filter_memberships_by_roles,
    get_django_user,
    set_preferred,
)
from postorius.views.generic import MailmanClientMixin


logger = logging.getLogger(__name__)


class UserPreferencesView(FormView, MailmanClientMixin):
    """Generic view for the logged-in user's various preferences."""

    form_class = UserPreferences

    #: Disabled delivery_status choices that a subscriber cannot set. This is
    #: different from what an admi is allowed to set.
    delivery_status_disabled_fields = ['by_moderator', 'by_bounces']

    def get_context_data(self, **kwargs):
        data = super(UserPreferencesView, self).get_context_data(**kwargs)
        data['mm_user'] = self.mm_user
        return data

    def get_form_kwargs(self):
        kwargs = super(UserPreferencesView, self).get_form_kwargs()
        kwargs['preferences'] = self._get_preferences()
        # Disable the choice of by_admin and by_bounces for a user.
        kwargs[
            'disabled_delivery_choices'
        ] = self.delivery_status_disabled_fields
        return kwargs

    def _set_view_attributes(self, request, *args, **kwargs):
        self.mm_user = get_mailman_user(request.user)

    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        self._set_view_attributes(request, *args, **kwargs)
        return super(UserPreferencesView, self).dispatch(
            request, *args, **kwargs
        )

    def form_valid(self, form):
        try:
            form.save()
        except HTTPError as e:
            messages.error(self.request, e.msg)
        if form.has_changed():
            messages.success(
                self.request, _('Your preferences have been updated.')
            )
        else:
            messages.info(self.request, _('Your preferences did not change.'))
        return super(UserPreferencesView, self).form_valid(form)


class UserMailmanSettingsView(UserPreferencesView):
    """The logged-in user's global Mailman preferences."""

    form_class = UserPreferences
    template_name = 'postorius/user/mailman_settings.html'
    success_url = reverse_lazy('user_mailmansettings')

    def _get_preferences(self):
        # Get the defaults and pre-populate so view shows them
        combinedpreferences = self._get_combined_preferences()
        for key in combinedpreferences:
            if key != 'self_link':
                self.mm_user.preferences[key] = combinedpreferences[key]

        # This is a bit of a hack so preferences behave as users expect
        # We probably don't want to save, only display here
        # but this means that whatever preferences the users see first are
        # the ones they have unless they explicitly change them
        self.mm_user.preferences.save()

        return self.mm_user.preferences

    def _get_combined_preferences(self):
        # Get layers of default preferences to match how they are applied
        # We ignore self_link as we don't want to over-write it
        defaultpreferences = get_mailman_client().preferences
        combinedpreferences = {}
        for key in defaultpreferences:
            if key != 'self_link':
                combinedpreferences[key] = defaultpreferences[key]

        # Clobber defaults with any preferences already set
        for key in self.mm_user.preferences:
            if key != 'self_link':
                combinedpreferences[key] = self.mm_user.preferences[key]

        return combinedpreferences


class UserAddressPreferencesView(UserPreferencesView):
    """The logged-in user's address-based Mailman Preferences."""

    template_name = 'postorius/user/address_preferences.html'
    success_url = reverse_lazy('user_address_preferences')

    def get_form_class(self):
        return formset_factory(
            UserPreferences, formset=UserPreferencesFormset, extra=0
        )

    def _get_preferences(self):
        return [address.preferences for address in self.mm_user.addresses]

    def _get_combined_preferences(self):
        # grab the default preferences
        defaultpreferences = get_mailman_client().preferences

        # grab your global preferences
        globalpreferences = self.mm_user.preferences

        # start a new combined preferences object
        combinedpreferences = []

        for address in self.mm_user.addresses:
            # make a per-address prefs object
            prefs = {}

            # initialize with default preferences
            for key in defaultpreferences:
                if key != 'self_link':
                    prefs[key] = defaultpreferences[key]

            # overwrite with user's global preferences
            for key in globalpreferences:
                if key != 'self_link':
                    prefs[key] = globalpreferences[key]

            # overwrite with address-specific preferences
            for key in address.preferences:
                if key != 'self_link':
                    prefs[key] = address.preferences[key]
            combinedpreferences.append(prefs)

            # put the combined preferences back on the original object
            for key in prefs:
                if key != 'self_link':
                    address.preferences[key] = prefs[key]

        return combinedpreferences

    def get_context_data(self, **kwargs):
        data = super(UserAddressPreferencesView, self).get_context_data(
            **kwargs
        )
        data['formset'] = data.pop('form')
        for form, address in list(
            zip(data['formset'].forms, self.mm_user.addresses)
        ):
            form.address = address
        return data


class UserListOptionsView(UserPreferencesView):
    """The logged-in user's subscription preferences."""

    form_class = UserPreferences
    template_name = 'postorius/user/list_options.html'

    def _get_subscription(self, member_id):
        subscription = None
        # We *could* use the find_members API, but then we'd have to
        # authenticate that the found subscription belongs to the currently
        # logged-in user otherwise. That might be a faster choice, but this
        # page isn't that slow right now.
        for s in self.mm_user.subscriptions:
            if s.role == 'member' and s.member_id == member_id:
                subscription = s
                break
        if not subscription:
            raise Http404(_('Subscription does not exist'))
        return subscription

    def _set_view_attributes(self, request, *args, **kwargs):
        super(UserListOptionsView, self)._set_view_attributes(
            request, *args, **kwargs
        )
        self.member_id = kwargs.get('member_id')
        self.subscription = self._get_subscription(self.member_id)
        self.mlist = List.objects.get_or_404(
            fqdn_listname=self.subscription.list_id
        )
        if (
            self.subscription.subscription_mode
            == SubscriptionMode.as_user.name
        ):
            self.subscriber = self.subscription.user.user_id
        else:
            self.subscriber = self.subscription.email

    def _get_preferences(self):
        return self.subscription.preferences

    def get_context_data(self, **kwargs):
        data = super(UserListOptionsView, self).get_context_data(**kwargs)
        data['mlist'] = self.mlist
        user_emails = (
            EmailAddress.objects.filter(user=self.request.user, verified=True)
            .order_by('email')
            .values_list('email', flat=True)
        )
        mm_user = get_mailman_user(self.request.user)
        primary_email = None
        if mm_user.preferred_address is None:
            primary_email = set_preferred(self.request.user, mm_user)
        else:
            primary_email = mm_user.preferred_address.email
        data['change_subscription_form'] = ChangeSubscriptionForm(
            user_emails,
            mm_user.user_id,
            primary_email,
            initial={
                'subscriber': self.subscriber,
                'member_id': self.member_id,
            },
        )
        return data

    def get_success_url(self):
        return reverse(
            'user_list_options', kwargs=dict(member_id=self.member_id)
        )


class UserSubscriptionPreferencesView(UserPreferencesView):
    """The logged-in user's subscription-based Mailman Preferences."""

    template_name = 'postorius/user/subscription_preferences.html'
    success_url = reverse_lazy('user_subscription_preferences')

    def _get_subscriptions(self):
        subscriptions = []
        for s in self.mm_user.subscriptions:
            if s.role != 'member':
                continue
            subscriptions.append(s)
        return subscriptions

    def _set_view_attributes(self, request, *args, **kwargs):
        super(UserSubscriptionPreferencesView, self)._set_view_attributes(
            request, *args, **kwargs
        )
        self.subscriptions = self._get_subscriptions()

    def get_form_class(self):
        return formset_factory(
            UserPreferences, formset=UserPreferencesFormset, extra=0
        )

    def _get_preferences(self):
        return [sub.preferences for sub in self.subscriptions]

    def _get_combined_preferences(self):
        # grab the default preferences
        defaultpreferences = get_mailman_client().preferences

        # grab your global preferences
        globalpreferences = self.mm_user.preferences

        # start a new combined preferences object
        combinedpreferences = []

        for sub in self.subscriptions:
            # make a per-address prefs object
            prefs = {}

            # initialize with default preferences
            for key in defaultpreferences:
                if key != 'self_link':
                    prefs[key] = defaultpreferences[key]

            # overwrite with user's global preferences
            for key in globalpreferences:
                if key != 'self_link':
                    prefs[key] = globalpreferences[key]

            # overwrite with address-based preferences
            # There is currently no better way to do this,
            # we may consider revisiting.
            addresspreferences = {}
            for address in self.mm_user.addresses:
                if sub.email == address.email:
                    addresspreferences = address.preferences

            for key in addresspreferences:
                if key != 'self_link':
                    prefs[key] = addresspreferences[key]

            # overwrite with subscription-specific preferences
            for key in sub.preferences:
                if key != 'self_link':
                    prefs[key] = sub.preferences[key]

            combinedpreferences.append(prefs)

        return combinedpreferences
        # return [sub.preferences for sub in self.subscriptions]

    def get_context_data(self, **kwargs):
        data = super(UserSubscriptionPreferencesView, self).get_context_data(
            **kwargs
        )
        data['formset'] = data.pop('form')
        for form, subscription in list(
            zip(data['formset'].forms, self.subscriptions)
        ):
            form.list_id = subscription.list_id
            form.member_id = subscription.member_id
            form.subscription_mode = subscription.subscription_mode
            form.address = subscription.address
        return data


@login_required
def user_subscriptions(request):
    """Shows the subscriptions of a user."""
    mm_user = get_mailman_user(request.user)
    memberships = [m for m in mm_user.subscriptions]
    return render(
        request,
        'postorius/user/subscriptions.html',
        {'memberships': memberships},
    )


@require_GET
@superuser_required
def list_users(request):
    """List of all users."""
    client = get_mailman_client()
    query = request.GET.get('q')

    if query:

        def _find_users(count, page):
            return client.find_users_page(query, count, page)

        find_method = _find_users
    else:
        find_method = client.get_user_page

    users = paginate(
        find_method,
        request.GET.get('page'),
        request.GET.get('count'),
        paginator_class=MailmanPaginator,
    )
    return render(
        request,
        'postorius/user/all.html',
        {'all_users': users, 'query': query},
    )


@superuser_required
@sensitive_post_parameters('password1', 'password2')
def manage_user(request, user_id):
    """Manage a single Mailman user view."""
    client = get_mailman_client()
    user = client.get_user(user_id)
    user_form = ManageUserForm(user=user)
    addr_formset = formset_factory(
        ManageAddressForm, formset=ManageAddressFormSet, extra=0
    )
    sub_formset = formset_factory(
        ManageMemberForm, formset=ManageMemberFormSet, extra=0
    )
    django_user = get_django_user(user)
    addresses = addr_formset(addresses=user.addresses)
    subscriptions = sub_formset(
        members=filter_memberships_by_roles(
            user.subscriptions,
            roles=['member', 'nonmember'],
        )
    )
    ownerships = filter_memberships_by_roles(
        user.subscriptions,
        roles=['owner', 'moderator'],
    )

    change_password = None
    if django_user is not None:
        change_password = AdminPasswordChangeForm(django_user)
        # The form always grabs focus so stop it from doing that unless there
        # is an error in the form submitted.
        change_password.fields['password1'].widget.attrs['autofocus'] = False

    if request.method == 'POST':
        # There are 4 forms in this view page, which one was submitted is
        # distinguished based on the name of the submit button.
        if 'address_form' in request.POST:
            # This is the 'addresses' form, built using addr_formset.
            addresses = addr_formset(request.POST, addresses=user.addresses)
            if addresses.is_valid():
                updated = addresses.save()
                if updated:
                    messages.success(
                        request,
                        _('Successfully updated addresses {}').format(
                            ', '.join(updated)
                        ),
                    )     # noqa: E501
        elif 'subs_form' in request.POST:
            # This is the 'subscriptions' form, built using sub_formset.
            subscriptions = sub_formset(
                request.POST, members=user.subscriptions
            )
            if subscriptions.is_valid():
                updated = subscriptions.save()
                if updated:
                    messages.success(
                        request,
                        _('Successfully updated memberships for {}').format(
                            ', '.join(updated)
                        ),
                    )  # noqa: E501
        elif 'user_form' in request.POST:
            # This is the 'user' form, built using ManageUserForm.
            user_form = ManageUserForm(request.POST, user=user)
            if user_form.is_valid():
                user_form.save()
                messages.success(request, _('Successfully updated user.'))
        elif 'change_password' in request.POST:
            change_password = AdminPasswordChangeForm(
                django_user, request.POST
            )
            if change_password.is_valid():
                change_password.save()
                # This will log the user out, which we want because the admin
                # is changing their password. In case of user changing their
                # own passowrd, they can remain authenticated.
                messages.success(request, _('Password updated successfully'))
                # Stop the form from grabbing the passowrd if successfully
                # updated.
                change_password.fields['password1'].widget.attrs[
                    'autofocus'
                ] = False

        return redirect(reverse('manage_user', args=(user_id,)))

    # In case of GET request, return the formsets with initial data.
    return render(
        request,
        'postorius/user/manage.html',
        {
            'auser': user,
            'user_form': user_form,
            'change_password': change_password,
            'django_user': django_user,
            'addresses': addresses,
            'subscriptions': subscriptions,
            'ownerships': ownerships,
        },
    )


@superuser_required
def delete_user(request, user_id):
    client = get_mailman_client()
    user = client.get_user(user_id)
    django_user = get_django_user(user)
    if request.method == 'POST':
        try:
            user.delete()
            if django_user:
                django_user.delete()
        except HTTPError as e:
            messages.error(request, e.msg)
        else:
            messages.success(request, _('Successfully deleted account'))
        return redirect(reverse('list_users'))
    return render(
        request, 'postorius/user/confirm_delete.html', {'auser': user}
    )



# -*- coding: utf-8 -*-
# Copyright (C) 2012-2023 by the Free Software Foundation, Inc.
#
# This file is part of Postorius.
#
# Postorius is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option)
# any later version.
#
# Postorius is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# Postorius.  If not, see <http://www.gnu.org/licenses/>.


from django import forms
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.core.validators import validate_email
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from postorius.forms.fields import SiteModelChoiceField


def _get_web_host_help():
    # Using a function is necessary, otherwise reverse() will be called before
    # URLConfs are loaded.
    return _(
        'The domain from which you want the web UI to be served from. '
        'This can be same or different from the Mail Host. '
        'You can edit the list of available web hosts <a href="%s">here</a>.'
    ) % reverse('admin:sites_site_changelist')


class DomainForm(forms.Form):
    """
    Form to add a domain.
    """

    mail_host = forms.CharField(
        label=_('Mail Host'),
        error_messages={
            'required': _('Please enter a domain name'),
            'invalid': _('Please enter a valid domain name.'),
        },
        required=True,
        help_text=_(
            'The domain for your mailing lists. For example when you want '
            'lists like testing@example.com, enter example.com here.'
        ),
    )
    description = forms.CharField(label=_('Description'), required=False)
    alias_domain = forms.CharField(
        label=_('Alias Domain'),
        error_messages={
            'invalid': _('Please enter a valid domain name or nothing.')
        },
        required=False,
        help_text=_('Normally empty.  Used only for unusual Postfix configs.'),
    )
    site = SiteModelChoiceField(
        label=_('Web Host'),
        error_messages={
            'required': _('Please enter a domain name'),
            'invalid': _('Please enter a valid domain name.'),
        },
        required=True,
        queryset=Site.objects.order_by('name').all(),
        initial=lambda: Site.objects.get_current(),
        help_text=_get_web_host_help,
    )

    def clean_mail_host(self):
        mail_host = self.cleaned_data['mail_host']
        try:
            validate_email('mail@' + mail_host)
        except ValidationError:
            raise forms.ValidationError(_('Please enter a valid domain name'))
        return mail_host

    def clean_alias_domain(self):
        alias_domain = self.cleaned_data['alias_domain']
        if alias_domain != '':
            try:
                validate_email('mail@' + alias_domain)
            except ValidationError:
                raise forms.ValidationError(
                    _('Please enter a valid domain name or nothing.')
                )
        return alias_domain


class DomainEditForm(DomainForm):
    """
    Form to edit domains
    separte from the DomainForm, so that the mail_host can't be changed.
    """

    mail_host = None


class DomainOwnerForm(forms.Form):
    """Form to add a owner for a domain."""

    email = forms.EmailField(
        label=_("Owner's Email"),
        required=True,
    )



# Generated by Django 2.2.11 on 2021-03-30 05:48

from django.db import migrations, models


# flake8: noqa

class Migration(migrations.Migration):

    dependencies = [
        ('postorius', '0013_auto_20201116_0058'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailtemplate',
            name='name',
            field=models.CharField(choices=[('list:admin:action:post', '[list:admin:action:post] - Sent to the list administrators when moderator approval for a posting is required.'), ('list:admin:action:subscribe', '[list:admin:action:subscribe] - Sent to the list administrators when moderator approval for a subscription request is required.'), ('list:admin:action:unsubscribe', '[list:admin:action:unsubscribe] - Sent to the list administrators when moderator approval for an unsubscription request is required.'), ('list:admin:notice:subscribe', '[list:admin:notice:subscribe] - Sent to the list administrators to notify them when a new member has been subscribed.'), ('list:admin:notice:unrecognized', '[list:admin:notice:unrecognized] - Sent to the list administrators when a bounce message in an unrecognized format has been received.'), ('list:admin:notice:increment', "[list:admin:notice:increment] - Sent to the list administrators to notify them when a member's bounce score is incremented."), ('list:admin:notice:disable', "[list:admin:notice:disable] - Sent to the list administrators to notify them when a member's delivery is disabled due to excessive bounces."), ('list:admin:notice:unsubscribe', '[list:admin:notice:unsubscribe] - Sent to the list administrators to notify them when a member has been unsubscribed.'), ('list:member:digest:footer', '[list:member:digest:footer] - The footer for a digest message.'), ('list:member:digest:header', '[list:member:digest:header] - The header for a digest message.'), ('list:member:digest:masthead', '[list:member:digest:masthead] - The digest “masthead”; i.e. a common introduction for all digest messages.'), ('list:member:regular:footer', '[list:member:regular:footer] - The footer for a regular (non-digest) message.'), ('list:member:regular:header', '[list:member:regular:header] - The header for a regular (non-digest) message.'), ('list:user:action:subscribe', '[list:user:action:subscribe] - The message sent to subscribers when a subscription confirmation is required.'), ('list:user:action:unsubscribe', '[list:user:action:unsubscribe] - The message sent to subscribers when an unsubscription confirmation is required.'), ('list:user:action:invite', '[list:user:action:invite] - The message sent to subscribers when they are invited to join a list.'), ('list:user:notice:goodbye', '[list:user:notice:goodbye] - The notice sent to a member when they unsubscribe from a mailing list.'), ('list:user:notice:hold', '[list:user:notice:hold] - The notice sent to a poster when their message is being held for moderator approval.'), ('list:user:notice:no-more-today', '[list:user:notice:no-more-today] - Sent to a user when the maximum number of autoresponses has been reached for that day.'), ('list:user:notice:post', '[list:user:notice:post] - Notice sent to a poster when their message has been received by the mailing list.'), ('list:user:notice:probe', '[list:user:notice:probe] - A bounce probe sent to a member when their subscription has been disabled due to bounces.'), ('list:user:notice:refuse', '[list:user:notice:refuse] - Notice sent to a poster when their message has been rejected by the list’s moderator.'), ('list:user:notice:rejected', '[list:user:notice:rejected] - Notice sent to a poster when their message has been rejected automatically.'), ('list:user:notice:welcome', '[list:user:notice:welcome] - The notice sent to a member when they are subscribed to the mailing list.'), ('list:user:notice:warning', '[list:user:notice:warning] - The notice sent to a member when their membership has been disabled due to excessive bounces.')], help_text='Choose the template you want to customize.', max_length=100),
        ),
    ]
